/*
 * $Id: rb_timer.c 40 2004-07-25 13:14:34Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>
#include <stdbool.h>

#include "rb_ecore.h"

typedef struct {
	Ecore_Timer *timer;
	void *cb;
	bool deleted;
} RbEcoreTimer;

static int on_timer (void *data)
{
	VALUE r;
	RbEcoreTimer *timer = data;

	r = rb_funcall ((VALUE) timer->cb, rb_intern ("call"), 0);

	/* if the callback returns false, we return 0 and Ecore
	 * will remove the timer
	 */
	if (r == Qfalse)
		timer->deleted = true;

	return (r != Qfalse);
}

static void c_free (RbEcoreTimer *timer)
{
	if (timer->timer && !timer->deleted)
		ecore_timer_del (timer->timer);

	ecore_shutdown ();

	free (timer);
}

static VALUE c_new (VALUE klass, VALUE interval)
{
	VALUE self;
	RbEcoreTimer *timer;

	if (!rb_block_given_p ())
		return Qnil;

	self = Data_Make_Struct (klass, RbEcoreTimer, NULL, c_free, timer);

	ecore_init ();

	timer->cb = (void *) rb_block_proc ();
	timer->timer = ecore_timer_add (NUM2DBL (interval),
	                                on_timer, timer);

	rb_obj_call_init (self, 0, NULL);

	return self;
}

static VALUE c_delete (VALUE self)
{
	RbEcoreTimer *timer = NULL;

	Data_Get_Struct (self, RbEcoreTimer, timer);

	if (timer->timer && !timer->deleted) {
		ecore_timer_del (timer->timer);
		timer->timer = NULL;
		timer->deleted = true;
	} else
		rb_raise (rb_eException, "Timer already deleted!");

	return Qnil;
}

void Init_Timer (void)
{
	VALUE c = rb_define_class_under (mEcore, "Timer", rb_cObject);

	rb_define_singleton_method (c, "new", c_new, 1);
	rb_define_method (c, "delete", c_delete, 0);
}
