/*
 * $Id: rb_job.c 40 2004-07-25 13:14:34Z tilman $
 *
 * Copyright (C) 2004 Tilman Sauerbeck (tilman at code-monkey de)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ruby.h>

#include <Ecore.h>
#include <Ecore_Job.h>
#include <stdbool.h>

#include "rb_ecore_job.h"

typedef struct {
	Ecore_Job *job;
	void *cb;
	bool deleted;
} RbEcoreJob;

static void on_job (void *data)
{
	RbEcoreJob *job = data;

	rb_funcall ((VALUE) job->cb, rb_intern ("call"), 0);
	job->deleted = true;
}

static void c_free (RbEcoreJob *job)
{
	if (job->job && !job->deleted)
		ecore_job_del (job->job);

	ecore_shutdown ();

	free (job);
}

static VALUE c_new (VALUE klass)
{
	VALUE self;
	RbEcoreJob *job;

	if (!rb_block_given_p ())
		return Qnil;

	self = Data_Make_Struct (klass, RbEcoreJob, NULL, c_free, job);

	ecore_init ();

	job->cb = (void *) rb_block_proc ();
	job->job = ecore_job_add (on_job, job);

	rb_obj_call_init (self, 0, NULL);

	return self;
}

static VALUE c_delete (VALUE self)
{
	VALUE ret = Qfalse;
	RbEcoreJob *job = NULL;

	Data_Get_Struct (self, RbEcoreJob, job);

	if (job->job && !job->deleted) {
		ecore_job_del (job->job);
		job->deleted = true;
		job->job = NULL;
		ret = Qtrue;
	}

	return ret;
}

void Init_Job (void)
{
	VALUE c = rb_define_class_under (mJob, "Job", rb_cObject);

	rb_define_singleton_method (c, "new", c_new, 0);
	rb_define_method (c, "delete", c_delete, 0);
}
